import middy, { MiddlewareObj } from '@middy/core';
import * as aws_lambda from 'aws-lambda';
import { APIGatewayProxyEvent, APIGatewayProxyResult, Context, ScheduledEvent } from 'aws-lambda';
import * as Joi from 'joi';
import { ObjectSchema } from 'joi';
import { BatchWriteCommandInput, BatchWriteCommandOutput, DeleteCommandInput, DeleteCommandOutput, GetCommandInput, GetCommandOutput, PutCommandInput, PutCommandOutput, QueryCommandInput, QueryCommandOutput, ScanCommandInput, ScanCommandOutput, UpdateCommandInput, UpdateCommandOutput } from '@aws-sdk/lib-dynamodb';
import * as type_fest from 'type-fest';

/**
 * The `HttpError` class extends `Error` providing additional, standardized attributes
 * for AWS Lambda functions. The attributes include: `statusCode`.
 */
declare class HttpError extends Error {
    name: string;
    statusCode: number;
    constructor(messageOrError: string | Error, statusCode: number);
}

/**
 * The `ServiceError` class extends `Error` providing additional, standardized attributes
 * for AWS Lambda functions. The attributes include: `code` and `statusCode`.
 */
declare class ServiceError extends Error {
    name: string;
    code: number;
    statusCode: number;
    constructor(messageOrError: string | Error, code?: number, statusCode?: number);
}

/**
 * `BadRequestError` extends `HttpError` creating a specialized Error class for the
 * Bad Request (400) HTTP response code.
 * @see {@link HttpError}
 */
declare class BadRequestError extends HttpError {
    name: string;
    constructor(message?: string);
}

/**
 * `Forbidden` extends `HttpError` creating a specialized Error class for the
 * Forbidden (403) HTTP response code.
 * @see {@link HttpError}
 */
declare class ForbiddenError extends HttpError {
    name: string;
    constructor(message?: string);
}

/**
 * `NotFoundError` extends `HttpError` creating a specialized Error class for the
 * Not Found (404) HTTP response code.
 * @see {@link HttpError}
 */
declare class NotFoundError extends HttpError {
    name: string;
    constructor(message?: string);
}

/**
 * `HttpErrorHandlerOptions` provide configuration to the `http-error-handler`
 * middleware.
 */
type HttpErrorHandlerOptions = {
    defaultMessage?: string;
    defaultStatusCode?: number;
    logger?(message: string): void;
};
/**
 * Create middleware to process errors thrown from AWS Lambda handler functions.
 * @param [opts] - Optional. `HttpErrorHandlerOptions` configuration.
 * @returns Middleware to process errors thrown handler functions.
 */
declare const httpErrorHandler: (opts?: HttpErrorHandlerOptions) => MiddlewareObj<APIGatewayProxyEvent, APIGatewayProxyResult>;

/**
 * ValidatorOptions
 */
type ValidatorOptions = {
    eventSchema?: Joi.ObjectSchema;
    logger?(message: string): void;
};
/**
 * Validate Lambda events with a Joi schema.
 * @param options - Options for the validator.
 * @returns Middleware to perform validation against a Joi schema.
 */
declare const validator: (options: ValidatorOptions) => MiddlewareObj;

/**
 * The `LambdaConfig` type describes the configuration values supplied to every AWS Lambda
 * function by default. Extend `LambdaConfig` with your own service-specific configuration
 * type.
 *
 * Example:
 * ```
 * type ServiceConfig = LambdaConfig & {
 *   FOO: string;
 *   BAR: number;
 * }
 * ```
 */
type LambdaConfig = {
    AWS_EXECUTION_ENV: string;
    AWS_LAMBDA_FUNCTION_NAME: string;
    AWS_LAMBDA_FUNCTION_MEMORY_SIZE: string;
    AWS_LAMBDA_FUNCTION_VERSION: string;
    AWS_REGION: string;
};
/**
 * Validates the configuration with the supplied Joi ObjectSchema. Uses `process.env` as the
 * configuration source.
 * @param schema - A `Joi.ObjectSchema` of type `TConfig` to validate the configuration values.
 * @returns An Object of type `TConfig` containing the validated configuration if successful,
 * otherwise throws a `ServiceError`.
 * @throws Throws a `ServiceError` when validation is unsuccessful.
 */
declare function validateConfig<TConfig>(schema: Joi.ObjectSchema<TConfig>): TConfig;
/**
 * A Joi ObjectSchema to validate the base AWS Lambda function configuration values.
 */
declare const lambdaConfigSchema: Joi.ObjectSchema<any>;
/**
 * A `LambdaConfig` object containing the base AWS Lambda function configuration values.
 * This is useful when your function does not have additional configuration attributes.
 * @see {@link LambdaConfig}
 */
declare const lambdaConfigValues: LambdaConfig;

/**
 * A `DynamoClient` provides operations for accessing and mutating one or more Items
 * within an AWS DynamoDB Table.
 */
type DynamoClient = {
    batchWriteItems: (input: BatchWriteCommandInput) => Promise<BatchWriteCommandOutput>;
    deleteItem: (input: DeleteCommandInput) => Promise<DeleteCommandOutput>;
    getItem: (input: GetCommandInput) => Promise<GetCommandOutput>;
    putItem: (input: PutCommandInput) => Promise<PutCommandOutput>;
    queryItems: (input: QueryCommandInput) => Promise<QueryCommandOutput>;
    scanItems: (input: ScanCommandInput) => Promise<ScanCommandOutput>;
    updateItem: (input: UpdateCommandInput) => Promise<UpdateCommandOutput>;
};
/**
 * Use the `DynamoService` to access or mutate Items within an AWS DynamoDB Table.
 */
declare const DynamoService: DynamoClient;

/**
 * The AWS Lambda handler function signature for API Gateway proxy events.
 */
type APIGatewayHandlerFn = (event: APIGatewayProxyEvent, context: Context) => Promise<APIGatewayProxyResult>;
/**
 * The AWS Lambda handler function signature for Scheduled events
 * (e.g. cron events from AWS EventBridge).
 */
type ScheduledHandlerFn = (event: ScheduledEvent, context: Context) => Promise<void>;
/**
 * Base options for `middyfy` functions.
 */
type MiddyfyOptions<THandler> = {
    handler: THandler;
    logger?: (message: string) => void;
};
/**
 * Options for middyfied API Gateway event handler functions.
 */
type APIGatewayMiddyfyOptions = MiddyfyOptions<APIGatewayHandlerFn> & {
    eventSchema?: ObjectSchema;
    defaultErrorMessage?: string;
    defaultErrorStatusCode?: number;
};
/**
 * Options for middyfied Scheduled event handler functions.
 */
type ScheduledMiddyfyOptions = MiddyfyOptions<ScheduledHandlerFn> & {
    eventSchema?: ObjectSchema;
};
/**
 * Wraps an AWS Gateway proxy event handler function in middleware, returning the
 * AWS Lambda handler function.
 * @param options - The `APIGatewayMiddyfyOptions` object.
 * @returns A middyfied handler function.
 */
declare const middyfyAPIGateway: (options: APIGatewayMiddyfyOptions) => middy.MiddyfiedHandler<APIGatewayProxyEvent & Omit<APIGatewayProxyEvent, "body"> & {
    body: type_fest.JsonValue;
} & {
    multiValueQueryStringParameters: aws_lambda.APIGatewayProxyEventMultiValueQueryStringParameters;
    pathParameters: aws_lambda.APIGatewayProxyEventPathParameters;
    queryStringParameters: aws_lambda.APIGatewayProxyEventQueryStringParameters;
}, APIGatewayProxyResult, Error, Context>;
/**
 * Wraps a Scheduled event handler function in middleware, returning the
 * AWS Lambda handler function.
 * @param options - The `ScheduledMiddyfyOptions` object.
 * @returns A middyfied handler function.
 */
declare const middyfyScheduled: (options: ScheduledMiddyfyOptions) => middy.MiddyfiedHandler<ScheduledEvent<any>, void, Error, Context>;

export { type APIGatewayHandlerFn, type APIGatewayMiddyfyOptions, BadRequestError, DynamoService, ForbiddenError, HttpError, type HttpErrorHandlerOptions, type LambdaConfig, type MiddyfyOptions, NotFoundError, type ScheduledHandlerFn, type ScheduledMiddyfyOptions, ServiceError, type ValidatorOptions, httpErrorHandler, lambdaConfigSchema, lambdaConfigValues, middyfyAPIGateway, middyfyScheduled, validateConfig, validator };
